/*
#  Copyright(C) 2015-2018  all rights reserved

#  This program is a free software; you can redistribute it and / or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
#  GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.gnu.org/Licenses/
*/

#include "StdAfx.h"
#include "Constants.h"

Constants::Constants(void)
{
	m_dPi = 3.14159265358979323846;
	//mass values
	m_dMASS_INITIAL = 18.015;
	m_dMASS_A = 71.0788;
	m_dMASS_C = 103.1388;
	m_dMASS_D = 115.0886;
	m_dMASS_E = 129.1155;
	m_dMASS_F = 147.1766;
	m_dMASS_G = 57.0519;
	m_dMASS_H = 137.1411;
	m_dMASS_I = 113.1594;
	m_dMASS_K = 128.1741;  
	m_dMASS_L = 113.1594;
	m_dMASS_M = 131.1926;
	m_dMASS_N = 114.104;
	m_dMASS_P = 97.1167;
	m_dMASS_Q = 128.1307;
	m_dMASS_R = 156.1875;
	m_dMASS_S = 87.0782;
	m_dMASS_T = 101.1051;
	m_dMASS_V = 99.1326;
	m_dMASS_W = 186.2132;
	m_dMASS_Y = 163.1760;

	//pKa values for amino acids
	m_dPKa_N_TERM = 9.69;
	m_dPKa_C_TERM = 2.34;
	m_dPKa_K = 10.5;
	m_dPKa_R = 12.4;
	m_dPKa_H = 6.0;
	m_dPKa_D = 3.86;
	m_dPKa_E = 4.25;
	m_dPKa_C = 8.33;
	m_dPKa_Y = 10.0;

	//atom-based hydrophobic moment
	//http://www.genome.ad.jp/dbget-bin/www_bget?aax1:EISD860102
	m_dATOM_BASED_HYDROPHOBIC_MOMENT_A = 0.00;
	m_dATOM_BASED_HYDROPHOBIC_MOMENT_C = 0.17;
	m_dATOM_BASED_HYDROPHOBIC_MOMENT_D = 1.90;
	m_dATOM_BASED_HYDROPHOBIC_MOMENT_E = 3.00;
	m_dATOM_BASED_HYDROPHOBIC_MOMENT_F = 1.10;
	m_dATOM_BASED_HYDROPHOBIC_MOMENT_G = 0.00;
	m_dATOM_BASED_HYDROPHOBIC_MOMENT_H = 0.99;
	m_dATOM_BASED_HYDROPHOBIC_MOMENT_I = 1.20;
	m_dATOM_BASED_HYDROPHOBIC_MOMENT_K = 5.70;
	m_dATOM_BASED_HYDROPHOBIC_MOMENT_L = 1.00;
	m_dATOM_BASED_HYDROPHOBIC_MOMENT_M = 1.90;
	m_dATOM_BASED_HYDROPHOBIC_MOMENT_N = 1.30;
	m_dATOM_BASED_HYDROPHOBIC_MOMENT_P = 0.18;
	m_dATOM_BASED_HYDROPHOBIC_MOMENT_Q = 1.90;
	m_dATOM_BASED_HYDROPHOBIC_MOMENT_R = 10.00;
	m_dATOM_BASED_HYDROPHOBIC_MOMENT_S = 0.73;
	m_dATOM_BASED_HYDROPHOBIC_MOMENT_T = 1.50;
	m_dATOM_BASED_HYDROPHOBIC_MOMENT_V = 0.48;
	m_dATOM_BASED_HYDROPHOBIC_MOMENT_W = 1.60;
	m_dATOM_BASED_HYDROPHOBIC_MOMENT_Y = 1.80;

	//Total propensity to be buried
	//http://www.genome.ad.jp/dbget-bin/www_bget?aax1:WERD780101
	m_dTOTAL_PROP_TO_BE_BURIED_A = 0.52;
	m_dTOTAL_PROP_TO_BE_BURIED_C = 0.83;
	m_dTOTAL_PROP_TO_BE_BURIED_D = 0.37;
	m_dTOTAL_PROP_TO_BE_BURIED_E = 0.38;
	m_dTOTAL_PROP_TO_BE_BURIED_F = 0.87;
	m_dTOTAL_PROP_TO_BE_BURIED_G = 0.41;
	m_dTOTAL_PROP_TO_BE_BURIED_H = 0.70;
	m_dTOTAL_PROP_TO_BE_BURIED_I = 0.79;
	m_dTOTAL_PROP_TO_BE_BURIED_K = 0.31;
	m_dTOTAL_PROP_TO_BE_BURIED_L = 0.77;
	m_dTOTAL_PROP_TO_BE_BURIED_M = 0.76;
	m_dTOTAL_PROP_TO_BE_BURIED_N = 0.42;
	m_dTOTAL_PROP_TO_BE_BURIED_P = 0.35;
	m_dTOTAL_PROP_TO_BE_BURIED_Q = 0.35;
	m_dTOTAL_PROP_TO_BE_BURIED_R = 0.49;
	m_dTOTAL_PROP_TO_BE_BURIED_S = 0.49;
	m_dTOTAL_PROP_TO_BE_BURIED_T = 0.38;
	m_dTOTAL_PROP_TO_BE_BURIED_V = 0.72;
	m_dTOTAL_PROP_TO_BE_BURIED_W = 0.86;
	m_dTOTAL_PROP_TO_BE_BURIED_Y = 0.64;
	
	//Hydropathy
	//AAIndex = YTJ820101
	//Hydropathy index (Kyte-Doolittle, 1982)
	//Kyte, J. and Doolittle, R.F.
	//A simple method for displaying the hydropathic character of a protein, J. Mol. Biol. 157, 105-132 (1982)
	m_dHYDROPATHY_A = 1.8;
	m_dHYDROPATHY_C = 2.5;
	m_dHYDROPATHY_D = -3.5;
	m_dHYDROPATHY_E = -3.5;
	m_dHYDROPATHY_F = 2.8;
	m_dHYDROPATHY_G = -0.4;
	m_dHYDROPATHY_H = -3.2;
	m_dHYDROPATHY_I = 4.5;
	m_dHYDROPATHY_K = -3.9;
	m_dHYDROPATHY_L = 3.8;
	m_dHYDROPATHY_M = 1.9;
	m_dHYDROPATHY_N = -3.5;
	m_dHYDROPATHY_P = -1.6;
	m_dHYDROPATHY_Q = -3.5;
	m_dHYDROPATHY_R = -4.5;
	m_dHYDROPATHY_S = -0.8;
	m_dHYDROPATHY_T = -0.7;
	m_dHYDROPATHY_V = 4.2;
	m_dHYDROPATHY_W = -0.9;
	m_dHYDROPATHY_Y = -1.3;

	//Normalized frequency of beta sheet (Chou-Fasman, 1978b)
	//AAIndex = CHOP780202
	//http://www.genome.jp/dbget-bin/www_bget?aaindex+CHOP780202
	m_dNORM_FREQ_BETA_SHEET_A = 0.83;
	m_dNORM_FREQ_BETA_SHEET_C = 1.19;
	m_dNORM_FREQ_BETA_SHEET_D = 0.54;
	m_dNORM_FREQ_BETA_SHEET_E = 0.37;
	m_dNORM_FREQ_BETA_SHEET_F = 1.38;
	m_dNORM_FREQ_BETA_SHEET_G = 0.75;
	m_dNORM_FREQ_BETA_SHEET_H = 0.87;
	m_dNORM_FREQ_BETA_SHEET_I = 1.60;
	m_dNORM_FREQ_BETA_SHEET_K = 0.74;
	m_dNORM_FREQ_BETA_SHEET_L = 1.30;
	m_dNORM_FREQ_BETA_SHEET_M = 1.05;
	m_dNORM_FREQ_BETA_SHEET_N = 0.89;
	m_dNORM_FREQ_BETA_SHEET_P = 0.55;
	m_dNORM_FREQ_BETA_SHEET_Q = 1.10;
	m_dNORM_FREQ_BETA_SHEET_R = 0.93;
	m_dNORM_FREQ_BETA_SHEET_S = 0.75;
	m_dNORM_FREQ_BETA_SHEET_T = 1.19;
	m_dNORM_FREQ_BETA_SHEET_V = 1.70;
	m_dNORM_FREQ_BETA_SHEET_W = 1.37;
	m_dNORM_FREQ_BETA_SHEET_Y = 1.47;

	//Normalized frequency of beta turn (Chou-Fasman, 1978b)
	//AAIndex = CHOP780203
	//http://www.genome.jp/dbget-bin/www_bget?aaindex+CHOP780203
	m_dNORM_FREQ_BETA_TURN_A = 0.74;
	m_dNORM_FREQ_BETA_TURN_C = 0.96;
	m_dNORM_FREQ_BETA_TURN_D = 1.52;
	m_dNORM_FREQ_BETA_TURN_E = 0.95;
	m_dNORM_FREQ_BETA_TURN_F = 0.66;
	m_dNORM_FREQ_BETA_TURN_G = 1.56;
	m_dNORM_FREQ_BETA_TURN_H = 0.95;
	m_dNORM_FREQ_BETA_TURN_I = 0.47;
	m_dNORM_FREQ_BETA_TURN_K = 1.19;
	m_dNORM_FREQ_BETA_TURN_L = 0.50;
	m_dNORM_FREQ_BETA_TURN_M = 0.60;
	m_dNORM_FREQ_BETA_TURN_N = 1.46;
	m_dNORM_FREQ_BETA_TURN_P = 1.56;
	m_dNORM_FREQ_BETA_TURN_Q = 0.96;
	m_dNORM_FREQ_BETA_TURN_R = 1.01;
	m_dNORM_FREQ_BETA_TURN_S = 1.43;
	m_dNORM_FREQ_BETA_TURN_T = 0.98;
	m_dNORM_FREQ_BETA_TURN_V = 0.59;
	m_dNORM_FREQ_BETA_TURN_W = 0.60;
	m_dNORM_FREQ_BETA_TURN_Y = 1.14;

	//Normalized frequency of alpha-helix (Chou-Fasman, 1978b)
	//AAIndex = CHOP780201
	//http://www.genome.jp/dbget-bin/www_bget?aaindex+CHOP780201
	m_dNORM_FREQ_ALPHA_HELIX_A = 1.42;
	m_dNORM_FREQ_ALPHA_HELIX_C = 0.70;
	m_dNORM_FREQ_ALPHA_HELIX_D = 1.01;
	m_dNORM_FREQ_ALPHA_HELIX_E = 1.51;
	m_dNORM_FREQ_ALPHA_HELIX_F = 1.13;
	m_dNORM_FREQ_ALPHA_HELIX_G = 0.57;
	m_dNORM_FREQ_ALPHA_HELIX_H = 1.00;
	m_dNORM_FREQ_ALPHA_HELIX_I = 1.08;
	m_dNORM_FREQ_ALPHA_HELIX_K = 1.16;
	m_dNORM_FREQ_ALPHA_HELIX_L = 1.21;
	m_dNORM_FREQ_ALPHA_HELIX_M = 1.45;
	m_dNORM_FREQ_ALPHA_HELIX_N = 0.67;
	m_dNORM_FREQ_ALPHA_HELIX_P = 0.57;
	m_dNORM_FREQ_ALPHA_HELIX_Q = 1.11;
	m_dNORM_FREQ_ALPHA_HELIX_R = 0.98;
	m_dNORM_FREQ_ALPHA_HELIX_S = 0.77;
	m_dNORM_FREQ_ALPHA_HELIX_T = 0.83;
	m_dNORM_FREQ_ALPHA_HELIX_V = 1.06;
	m_dNORM_FREQ_ALPHA_HELIX_W = 1.08;
	m_dNORM_FREQ_ALPHA_HELIX_Y = 0.69;


	//Principal property value z3 (Wold et al., 1987)
	//AAIndex = WOLS870103
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:WOLS870103
	m_dPRINCIPAL_PROPERTY_VALUE_Z3_A = 0.09;
	m_dPRINCIPAL_PROPERTY_VALUE_Z3_C = 4.13;
	m_dPRINCIPAL_PROPERTY_VALUE_Z3_D = 2.36;
	m_dPRINCIPAL_PROPERTY_VALUE_Z3_E = -0.07;
	m_dPRINCIPAL_PROPERTY_VALUE_Z3_F = 0.45;
	m_dPRINCIPAL_PROPERTY_VALUE_Z3_G = 0.30;
	m_dPRINCIPAL_PROPERTY_VALUE_Z3_H = 1.11;
	m_dPRINCIPAL_PROPERTY_VALUE_Z3_I = -1.03;
	m_dPRINCIPAL_PROPERTY_VALUE_Z3_K = -3.14;
	m_dPRINCIPAL_PROPERTY_VALUE_Z3_L = -0.98;
	m_dPRINCIPAL_PROPERTY_VALUE_Z3_M = -0.41;
	m_dPRINCIPAL_PROPERTY_VALUE_Z3_N = 0.84;
	m_dPRINCIPAL_PROPERTY_VALUE_Z3_P = 2.23;
	m_dPRINCIPAL_PROPERTY_VALUE_Z3_Q = -1.14;
	m_dPRINCIPAL_PROPERTY_VALUE_Z3_R = -3.44;
	m_dPRINCIPAL_PROPERTY_VALUE_Z3_S = 0.57;
	m_dPRINCIPAL_PROPERTY_VALUE_Z3_T = -1.40;
	m_dPRINCIPAL_PROPERTY_VALUE_Z3_V = -1.29;
	m_dPRINCIPAL_PROPERTY_VALUE_Z3_W = 0.85;
	m_dPRINCIPAL_PROPERTY_VALUE_Z3_Y = 0.01;
	
	//Helix termination parameter at posision j+1 (Finkelstein et al., 1991)
	//AAINDEX = FINA910104 
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:FINA910104
	m_dHELIX_TERMINATION_PARA_AT_POSI_JPLUS1_A = 1.;
	m_dHELIX_TERMINATION_PARA_AT_POSI_JPLUS1_C = 1.;
	m_dHELIX_TERMINATION_PARA_AT_POSI_JPLUS1_D = 0.70;
	m_dHELIX_TERMINATION_PARA_AT_POSI_JPLUS1_E = 0.70;
	m_dHELIX_TERMINATION_PARA_AT_POSI_JPLUS1_F = 1.;
	m_dHELIX_TERMINATION_PARA_AT_POSI_JPLUS1_G = 1.5;
	m_dHELIX_TERMINATION_PARA_AT_POSI_JPLUS1_H = 1.;
	m_dHELIX_TERMINATION_PARA_AT_POSI_JPLUS1_I = 1.;
	m_dHELIX_TERMINATION_PARA_AT_POSI_JPLUS1_K = 1.70;
	m_dHELIX_TERMINATION_PARA_AT_POSI_JPLUS1_L = 1.;
	m_dHELIX_TERMINATION_PARA_AT_POSI_JPLUS1_M = 1.;
	m_dHELIX_TERMINATION_PARA_AT_POSI_JPLUS1_N = 1.;
	m_dHELIX_TERMINATION_PARA_AT_POSI_JPLUS1_P = 0.10;
	m_dHELIX_TERMINATION_PARA_AT_POSI_JPLUS1_Q = 1.;
	m_dHELIX_TERMINATION_PARA_AT_POSI_JPLUS1_R = 1.70;
	m_dHELIX_TERMINATION_PARA_AT_POSI_JPLUS1_S = 1.;
	m_dHELIX_TERMINATION_PARA_AT_POSI_JPLUS1_T = 1.;
	m_dHELIX_TERMINATION_PARA_AT_POSI_JPLUS1_V = 1.;
	m_dHELIX_TERMINATION_PARA_AT_POSI_JPLUS1_W = 1.;
	m_dHELIX_TERMINATION_PARA_AT_POSI_JPLUS1_Y = 1.;

	//Buriability
	//AAINDEX = ZHOH040103 
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:ZHOH040103
	BURIABILITY_A = 13.4;
	BURIABILITY_C = 22.6;
	BURIABILITY_D = 8.2;
	BURIABILITY_E = 7.3;
	BURIABILITY_F = 23.9;
	BURIABILITY_G = 7.0;
	BURIABILITY_H = 11.3;
	BURIABILITY_I = 20.3;
	BURIABILITY_K = 6.1;
	BURIABILITY_L = 20.8;
	BURIABILITY_M = 15.7;
	BURIABILITY_N = 7.6;
	BURIABILITY_P = 9.9;
	BURIABILITY_Q = 8.5;
	BURIABILITY_R = 8.5;
	BURIABILITY_S = 8.2;
	BURIABILITY_T = 10.3;
	BURIABILITY_V = 19.5;
	BURIABILITY_W = 24.5;
	BURIABILITY_Y = 19.5;

	//Number of hydrogen bond donors (Fauchere et al., 1988)
	//AAINDEX = FAUJ880109
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:FAUJ880109
	NUMBER_OF_HYDROGEN_BOND_DONORS_A = 0.;
	NUMBER_OF_HYDROGEN_BOND_DONORS_C = 0.;
	NUMBER_OF_HYDROGEN_BOND_DONORS_D = 1.;
	NUMBER_OF_HYDROGEN_BOND_DONORS_E = 1.;
	NUMBER_OF_HYDROGEN_BOND_DONORS_F = 0.;
	NUMBER_OF_HYDROGEN_BOND_DONORS_G = 0.;
	NUMBER_OF_HYDROGEN_BOND_DONORS_H = 1.;
	NUMBER_OF_HYDROGEN_BOND_DONORS_I = 0.;
	NUMBER_OF_HYDROGEN_BOND_DONORS_K = 2.;
	NUMBER_OF_HYDROGEN_BOND_DONORS_L = 0.;
	NUMBER_OF_HYDROGEN_BOND_DONORS_M = 0.;
	NUMBER_OF_HYDROGEN_BOND_DONORS_N = 2.;
	NUMBER_OF_HYDROGEN_BOND_DONORS_P = 0.;
	NUMBER_OF_HYDROGEN_BOND_DONORS_Q = 2.;
	NUMBER_OF_HYDROGEN_BOND_DONORS_R = 4.;
	NUMBER_OF_HYDROGEN_BOND_DONORS_S = 1.;
	NUMBER_OF_HYDROGEN_BOND_DONORS_T = 1.;
	NUMBER_OF_HYDROGEN_BOND_DONORS_V = 0.;
	NUMBER_OF_HYDROGEN_BOND_DONORS_W = 1.;
	NUMBER_OF_HYDROGEN_BOND_DONORS_Y = 1.;

	//Relative preference value at N5 (Richardson-Richardson, 1988)
	//AAINDEX = RICJ880108
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:RICJ880108
	RELATIVE_PREFERENCE_VALUE_AT_N5_A = 1.4;
	RELATIVE_PREFERENCE_VALUE_AT_N5_C = 1.6;
	RELATIVE_PREFERENCE_VALUE_AT_N5_D = 0.6;
	RELATIVE_PREFERENCE_VALUE_AT_N5_E = 0.9;
	RELATIVE_PREFERENCE_VALUE_AT_N5_F = 1.;
	RELATIVE_PREFERENCE_VALUE_AT_N5_G = 0.6;
	RELATIVE_PREFERENCE_VALUE_AT_N5_H = 0.9;
	RELATIVE_PREFERENCE_VALUE_AT_N5_I = 0.9;
	RELATIVE_PREFERENCE_VALUE_AT_N5_K = 1.9;
	RELATIVE_PREFERENCE_VALUE_AT_N5_L = 1.1;
	RELATIVE_PREFERENCE_VALUE_AT_N5_M = 1.7;
	RELATIVE_PREFERENCE_VALUE_AT_N5_N = 1.2;
	RELATIVE_PREFERENCE_VALUE_AT_N5_P = 0.3;
	RELATIVE_PREFERENCE_VALUE_AT_N5_Q = 1.4;
	RELATIVE_PREFERENCE_VALUE_AT_N5_R = 1.2;
	RELATIVE_PREFERENCE_VALUE_AT_N5_S = 1.1;
	RELATIVE_PREFERENCE_VALUE_AT_N5_T = 0.6;
	RELATIVE_PREFERENCE_VALUE_AT_N5_V = 0.8;
	RELATIVE_PREFERENCE_VALUE_AT_N5_W = 1.4;
	RELATIVE_PREFERENCE_VALUE_AT_N5_Y = 0.2;

	//STERIMOL length of the side chain (Fauchere et al., 1988)
	//AAINDEX = FAUJ880104
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:FAUJ880104
	STERIMOL_LENGTH_OF_THE_SIDE_CHAIN_A = 2.87;
	STERIMOL_LENGTH_OF_THE_SIDE_CHAIN_C = 4.47;
	STERIMOL_LENGTH_OF_THE_SIDE_CHAIN_D = 4.74;
	STERIMOL_LENGTH_OF_THE_SIDE_CHAIN_E = 5.97;
	STERIMOL_LENGTH_OF_THE_SIDE_CHAIN_F = 4.62;
	STERIMOL_LENGTH_OF_THE_SIDE_CHAIN_G = 2.06;
	STERIMOL_LENGTH_OF_THE_SIDE_CHAIN_H = 5.23;
	STERIMOL_LENGTH_OF_THE_SIDE_CHAIN_I = 4.92;
	STERIMOL_LENGTH_OF_THE_SIDE_CHAIN_K = 6.89;
	STERIMOL_LENGTH_OF_THE_SIDE_CHAIN_L = 4.92;
	STERIMOL_LENGTH_OF_THE_SIDE_CHAIN_M = 6.36;
	STERIMOL_LENGTH_OF_THE_SIDE_CHAIN_N = 4.58;
	STERIMOL_LENGTH_OF_THE_SIDE_CHAIN_P = 4.11;
	STERIMOL_LENGTH_OF_THE_SIDE_CHAIN_Q = 6.11;
	STERIMOL_LENGTH_OF_THE_SIDE_CHAIN_R = 7.82;
	STERIMOL_LENGTH_OF_THE_SIDE_CHAIN_S = 3.97;
	STERIMOL_LENGTH_OF_THE_SIDE_CHAIN_T = 4.11;
	STERIMOL_LENGTH_OF_THE_SIDE_CHAIN_V = 4.11;
	STERIMOL_LENGTH_OF_THE_SIDE_CHAIN_W = 7.68;
	STERIMOL_LENGTH_OF_THE_SIDE_CHAIN_Y = 4.73;

	//Weights for alpha-helix at the window position of 6 (Qian-Sejnowski, 1988)
	//AAINDEX = QIAN880113
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:QIAN880113
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_6_A = -0.08;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_6_C = -0.25;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_6_D = -0.24;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_6_E = -0.19;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_6_F = 0.00;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_6_G = -0.10;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_6_H = 0.29;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_6_I = -0.01;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_6_K = 0.45;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_6_L = 0.28;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_6_M = 0.11;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_6_N = -0.08;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_6_P = -0.42;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_6_Q = -0.28;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_6_R = 0.05;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_6_S = 0.07;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_6_T = -0.33;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_6_V = -0.13;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_6_W = 0.36;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_6_Y = 0.00;

	// Relative preference value at C2 (Richardson-Richardson, 1988)
	//AAINDEX = RICJ880113
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:RICJ880113
	RELATIVE_PREFERENCE_VALUE_AT_C2_A = 1.4;
	RELATIVE_PREFERENCE_VALUE_AT_C2_C = 1.2;
	RELATIVE_PREFERENCE_VALUE_AT_C2_D = 0.7;
	RELATIVE_PREFERENCE_VALUE_AT_C2_E = 1.7;
	RELATIVE_PREFERENCE_VALUE_AT_C2_F = 0.3;
	RELATIVE_PREFERENCE_VALUE_AT_C2_G = 0.2;
	RELATIVE_PREFERENCE_VALUE_AT_C2_H = 1.8;
	RELATIVE_PREFERENCE_VALUE_AT_C2_I = 0.4;
	RELATIVE_PREFERENCE_VALUE_AT_C2_K = 1.9;
	RELATIVE_PREFERENCE_VALUE_AT_C2_L = 0.8;
	RELATIVE_PREFERENCE_VALUE_AT_C2_M = 1.3;
	RELATIVE_PREFERENCE_VALUE_AT_C2_N = 0.9;
	RELATIVE_PREFERENCE_VALUE_AT_C2_P = 0.2;
	RELATIVE_PREFERENCE_VALUE_AT_C2_Q = 1.6;
	RELATIVE_PREFERENCE_VALUE_AT_C2_R = 2.1;
	RELATIVE_PREFERENCE_VALUE_AT_C2_S = 1.6;
	RELATIVE_PREFERENCE_VALUE_AT_C2_T = 0.9;
	RELATIVE_PREFERENCE_VALUE_AT_C2_V = 0.7;
	RELATIVE_PREFERENCE_VALUE_AT_C2_W = 0.4;
	RELATIVE_PREFERENCE_VALUE_AT_C2_Y = 0.3;

	//Weights for alpha-helix at the window position of -6 (Qian-Sejnowski, 1988)
	//QIAN880101 
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:QIAN880101
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_MINUS6_A = 0.12;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_MINUS6_C = -0.25;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_MINUS6_D = 0.01;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_MINUS6_E = -0.02;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_MINUS6_F = 0.05;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_MINUS6_G = -0.02;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_MINUS6_H = -0.06;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_MINUS6_I = -0.07;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_MINUS6_K = 0.26;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_MINUS6_L = 0.05;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_MINUS6_M = 0.00;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_MINUS6_N = -0.10;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_MINUS6_P = -0.19;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_MINUS6_Q = -0.03;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_MINUS6_R = 0.04;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_MINUS6_S = -0.19;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_MINUS6_T = -0.04;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_MINUS6_V = -0.03;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_MINUS6_W = -0.06;
	WEIGHTS_FOR_ALPHA_HELIX_AT_THE_WINDOW_POSITION_OF_MINUS6_Y = -0.14;

	//Hydrophobicity index, 3.0 pH (Cowan-Whittaker, 1990)
	//COWR900101
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:COWR900101
	HYDROPHOBICITY_INDEX_3DOT0_PH_A = 0.42;
	HYDROPHOBICITY_INDEX_3DOT0_PH_C = 0.84;
	HYDROPHOBICITY_INDEX_3DOT0_PH_D = -0.51;
	HYDROPHOBICITY_INDEX_3DOT0_PH_E = -0.37;
	HYDROPHOBICITY_INDEX_3DOT0_PH_F = 1.74;
	HYDROPHOBICITY_INDEX_3DOT0_PH_G = 0.00;
	HYDROPHOBICITY_INDEX_3DOT0_PH_H = -2.28;
	HYDROPHOBICITY_INDEX_3DOT0_PH_I = 1.81;
	HYDROPHOBICITY_INDEX_3DOT0_PH_K = -2.03;
	HYDROPHOBICITY_INDEX_3DOT0_PH_L = 1.80;
	HYDROPHOBICITY_INDEX_3DOT0_PH_M = 1.18;
	HYDROPHOBICITY_INDEX_3DOT0_PH_N = -1.03;
	HYDROPHOBICITY_INDEX_3DOT0_PH_P = 0.86;
	HYDROPHOBICITY_INDEX_3DOT0_PH_Q = -0.96;
	HYDROPHOBICITY_INDEX_3DOT0_PH_R = -1.56;
	HYDROPHOBICITY_INDEX_3DOT0_PH_S = -0.64;
	HYDROPHOBICITY_INDEX_3DOT0_PH_T = -0.26;
	HYDROPHOBICITY_INDEX_3DOT0_PH_V = 1.34;
	HYDROPHOBICITY_INDEX_3DOT0_PH_W = 1.46;
	HYDROPHOBICITY_INDEX_3DOT0_PH_Y = 0.51;

	//Information measure for C-terminal helix (Robson-Suzuki, 1976)
	//AAINDEX = ROBB760104
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:ROBB760104
	INFO_MEASURE_FOR_C_TERMINAL_HELIX_A = 2.3;
	INFO_MEASURE_FOR_C_TERMINAL_HELIX_C = 6.1;
	INFO_MEASURE_FOR_C_TERMINAL_HELIX_D = -4.4;
	INFO_MEASURE_FOR_C_TERMINAL_HELIX_E = 2.5;
	INFO_MEASURE_FOR_C_TERMINAL_HELIX_F = 1.6;
	INFO_MEASURE_FOR_C_TERMINAL_HELIX_G = -8.3;
	INFO_MEASURE_FOR_C_TERMINAL_HELIX_H = 5.9;
	INFO_MEASURE_FOR_C_TERMINAL_HELIX_I = -0.5;
	INFO_MEASURE_FOR_C_TERMINAL_HELIX_K = 7.3;
	INFO_MEASURE_FOR_C_TERMINAL_HELIX_L = 0.1;
	INFO_MEASURE_FOR_C_TERMINAL_HELIX_M = 3.5;
	INFO_MEASURE_FOR_C_TERMINAL_HELIX_N = -3.3;
	INFO_MEASURE_FOR_C_TERMINAL_HELIX_P = -24.4;
	INFO_MEASURE_FOR_C_TERMINAL_HELIX_Q = 2.7;
	INFO_MEASURE_FOR_C_TERMINAL_HELIX_R = 1.4;
	INFO_MEASURE_FOR_C_TERMINAL_HELIX_S = -1.9;
	INFO_MEASURE_FOR_C_TERMINAL_HELIX_T = -3.7;
	INFO_MEASURE_FOR_C_TERMINAL_HELIX_V = 2.3;
	INFO_MEASURE_FOR_C_TERMINAL_HELIX_W = -0.9;
	INFO_MEASURE_FOR_C_TERMINAL_HELIX_Y = -0.6;

	//Weights for coil at the window position of -6 (Qian-Sejnowski, 1988)
	//QIAN880127
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:QIAN880127
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS6_A = -0.05;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS6_C = 0.30;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS6_D = 0.15;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS6_E = -0.02;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS6_F = 0.09;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS6_G = -0.14;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS6_H = -0.07;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS6_I = 0.26;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS6_K = -0.42;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS6_L = 0.04;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS6_M = 0.25;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS6_N = 0.00;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS6_P = 0.31;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS6_Q = -0.08;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS6_R = 0.06;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS6_S = -0.11;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS6_T = -0.06;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS6_V = 0.04;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS6_W = 0.19;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS6_Y = 0.33;

	//Hydrophobicity coefficient in RP-HPLC, C4 with 0.1%TFA/MeCN/H2O (Wilce et al. 1995)
	//AAINDEX = WILM950103
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:WILM950103
	HYDROPHOBICITY_COEFFICIENT_IN_RP_HPLC_A = -1.64;
	HYDROPHOBICITY_COEFFICIENT_IN_RP_HPLC_C = 9.30;
	HYDROPHOBICITY_COEFFICIENT_IN_RP_HPLC_D = 0.70;
	HYDROPHOBICITY_COEFFICIENT_IN_RP_HPLC_E = 1.18;
	HYDROPHOBICITY_COEFFICIENT_IN_RP_HPLC_F = -1.36;
	HYDROPHOBICITY_COEFFICIENT_IN_RP_HPLC_G = -1.85;
	HYDROPHOBICITY_COEFFICIENT_IN_RP_HPLC_H = 7.17;
	HYDROPHOBICITY_COEFFICIENT_IN_RP_HPLC_I = 3.02;
	HYDROPHOBICITY_COEFFICIENT_IN_RP_HPLC_K = -2.36;
	HYDROPHOBICITY_COEFFICIENT_IN_RP_HPLC_L = 0.83;
	HYDROPHOBICITY_COEFFICIENT_IN_RP_HPLC_M = 4.26;
	HYDROPHOBICITY_COEFFICIENT_IN_RP_HPLC_N = 0.83;
	HYDROPHOBICITY_COEFFICIENT_IN_RP_HPLC_P = 3.12;
	HYDROPHOBICITY_COEFFICIENT_IN_RP_HPLC_Q = -0.04;
	HYDROPHOBICITY_COEFFICIENT_IN_RP_HPLC_R = -3.28;
	HYDROPHOBICITY_COEFFICIENT_IN_RP_HPLC_S = 1.59;
	HYDROPHOBICITY_COEFFICIENT_IN_RP_HPLC_T = 2.31;
	HYDROPHOBICITY_COEFFICIENT_IN_RP_HPLC_V = 0.52;
	HYDROPHOBICITY_COEFFICIENT_IN_RP_HPLC_W = 2.61;
	HYDROPHOBICITY_COEFFICIENT_IN_RP_HPLC_Y = 2.37;

	//Fraction of site occupied by water (Krigbaum-Komoriya, 1979)
	//AAINDEX = KRIW790102 
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:KRIW790102
	FRACTION_OF_SITE_OCCUPIED_BY_WATER_A = 0.28;
	FRACTION_OF_SITE_OCCUPIED_BY_WATER_C = 0.11;
	FRACTION_OF_SITE_OCCUPIED_BY_WATER_D = 0.33;
	FRACTION_OF_SITE_OCCUPIED_BY_WATER_E = 0.37;
	FRACTION_OF_SITE_OCCUPIED_BY_WATER_F = 0.10;
	FRACTION_OF_SITE_OCCUPIED_BY_WATER_G = 0.28;
	FRACTION_OF_SITE_OCCUPIED_BY_WATER_H = 0.23;
	FRACTION_OF_SITE_OCCUPIED_BY_WATER_I = 0.12;
	FRACTION_OF_SITE_OCCUPIED_BY_WATER_K = 0.59;
	FRACTION_OF_SITE_OCCUPIED_BY_WATER_L = 0.16;
	FRACTION_OF_SITE_OCCUPIED_BY_WATER_M = 0.08;
	FRACTION_OF_SITE_OCCUPIED_BY_WATER_N = 0.31;
	FRACTION_OF_SITE_OCCUPIED_BY_WATER_P = 0.46;
	FRACTION_OF_SITE_OCCUPIED_BY_WATER_Q = 0.39;
	FRACTION_OF_SITE_OCCUPIED_BY_WATER_R = 0.34;
	FRACTION_OF_SITE_OCCUPIED_BY_WATER_S = 0.27;
	FRACTION_OF_SITE_OCCUPIED_BY_WATER_T = 0.26;
	FRACTION_OF_SITE_OCCUPIED_BY_WATER_V = 0.22;
	FRACTION_OF_SITE_OCCUPIED_BY_WATER_W = 0.15;
	FRACTION_OF_SITE_OCCUPIED_BY_WATER_Y = 0.25;

	//Normalized positional residue frequency at helix termini C2 (Aurora-Rose, 1998)
	//AAINDEX = AURR980114
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:AURR980114
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_C2_A = 1.33;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_C2_C = 0.44;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_C2_D = 0.60;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_C2_E = 1.43;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_C2_F = 1.22;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_C2_G = 0.20;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_C2_H = 1.02;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_C2_I = 1.58;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_C2_K = 1.71;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_C2_L = 1.63;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_C2_M = 1.76;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_C2_N = 0.64;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_C2_P = 0.07;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_C2_Q = 1.37;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_C2_R = 1.39;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_C2_S = 0.42;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_C2_T = 0.57;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_C2_V = 1.08;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_C2_W = 1.00;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_C2_Y = 1.02;

	//Normalized positional residue frequency at helix termini N'(Aurora-Rose, 1998)
	//AAINDEX = AURR980104
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:AURR980104
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_N_A = 0.75;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_N_C = 0.66;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_N_D = 1.72;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_N_E = 1.10;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_N_F = 0.88;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_N_G = 1.14;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_N_H = 0.96;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_N_I = 0.80;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_N_K = 0.66;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_N_L = 1.01;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_N_M = 1.02;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_N_N = 1.24;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_N_P = 1.33;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_N_Q = 1.08;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_N_R = 0.90;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_N_S = 1.20;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_N_T = 1.13;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_N_V = 0.58;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_N_W = 0.68;
	NORMALIZED_POSITIONAL_RESIDUE_FREQUENCY_AT_HELIX_TERMINI_N_Y = 0.80;

	//Optical rotation (Fasman, 1976)
	//AAINDEX = FASG760103
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:FASG760103
	OPTICAL_ROTATION_A = 1.80;
	OPTICAL_ROTATION_C = -16.50;
	OPTICAL_ROTATION_D = 5.05;
	OPTICAL_ROTATION_E = 12.0;
	OPTICAL_ROTATION_F = -34.50;
	OPTICAL_ROTATION_G = 0.0;
	OPTICAL_ROTATION_H = -38.50;
	OPTICAL_ROTATION_I = 12.40;
	OPTICAL_ROTATION_K = 14.60;
	OPTICAL_ROTATION_L = -11.0;
	OPTICAL_ROTATION_M = -10.0;
	OPTICAL_ROTATION_N = -5.6;
	OPTICAL_ROTATION_P = -86.20;
	OPTICAL_ROTATION_Q = 6.30;
	OPTICAL_ROTATION_R = 12.50;
	OPTICAL_ROTATION_S = -7.50;
	OPTICAL_ROTATION_T = -28.0;
	OPTICAL_ROTATION_V = 5.63;
	OPTICAL_ROTATION_W = -33.7;
	OPTICAL_ROTATION_Y = -10.0;

	//Optimized transfer energy parameter (Oobatake et al., 1985)
	//OOBM850103
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:OOBM850103
	OPTIMIZED_TRANSFER_ENERGY_PARAMETER_A = 0.46;
	OPTIMIZED_TRANSFER_ENERGY_PARAMETER_C = 0.20;
	OPTIMIZED_TRANSFER_ENERGY_PARAMETER_D = -0.33;
	OPTIMIZED_TRANSFER_ENERGY_PARAMETER_E = 0.48;
	OPTIMIZED_TRANSFER_ENERGY_PARAMETER_F = 0.52;
	OPTIMIZED_TRANSFER_ENERGY_PARAMETER_G = 0.64;
	OPTIMIZED_TRANSFER_ENERGY_PARAMETER_H = -1.31;
	OPTIMIZED_TRANSFER_ENERGY_PARAMETER_I = 3.28;
	OPTIMIZED_TRANSFER_ENERGY_PARAMETER_K = -1.71;
	OPTIMIZED_TRANSFER_ENERGY_PARAMETER_L = 0.43;
	OPTIMIZED_TRANSFER_ENERGY_PARAMETER_M = 0.15;
	OPTIMIZED_TRANSFER_ENERGY_PARAMETER_N = 1.31;
	OPTIMIZED_TRANSFER_ENERGY_PARAMETER_P = -0.58;
	OPTIMIZED_TRANSFER_ENERGY_PARAMETER_Q = -1.12;
	OPTIMIZED_TRANSFER_ENERGY_PARAMETER_R = -1.54;
	OPTIMIZED_TRANSFER_ENERGY_PARAMETER_S = -0.83;
	OPTIMIZED_TRANSFER_ENERGY_PARAMETER_T = -1.52;
	OPTIMIZED_TRANSFER_ENERGY_PARAMETER_V = 0.54;
	OPTIMIZED_TRANSFER_ENERGY_PARAMETER_W = 1.25;
	OPTIMIZED_TRANSFER_ENERGY_PARAMETER_Y = -2.21;

	//Weights for coil at the window position of 5 (Qian-Sejnowski, 1988)
	//AAINDEX = QIAN880138
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:QIAN880138
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_5_A = -0.23;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_5_C = 0.19;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_5_D = 0.34;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_5_E = 0.28;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_5_F = -0.02;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_5_G = 0.14;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_5_H = -0.22;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_5_I = 0.42;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_5_K = -0.15;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_5_L = -0.36;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_5_M = -0.19;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_5_N = 0.06;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_5_P = 0.78;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_5_Q = 0.47;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_5_R = -0.20;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_5_S = -0.29;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_5_T = -0.30;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_5_V = 0.29;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_5_W = -0.18;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_5_Y = 0.07;

	//Relative preference value at C1 (Richardson-Richardson, 1988)
	//AAINDEX = RICJ880114
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:RICJ880114
	RELATIVE_PREFERENCE_VALUE_AT_C1_A = 1.1;
	RELATIVE_PREFERENCE_VALUE_AT_C1_C = 1.6;
	RELATIVE_PREFERENCE_VALUE_AT_C1_D = 0.4;
	RELATIVE_PREFERENCE_VALUE_AT_C1_E = 0.8;
	RELATIVE_PREFERENCE_VALUE_AT_C1_F = 0.7;
	RELATIVE_PREFERENCE_VALUE_AT_C1_G = 0.2;
	RELATIVE_PREFERENCE_VALUE_AT_C1_H = 3.4;
	RELATIVE_PREFERENCE_VALUE_AT_C1_I = 0.7;
	RELATIVE_PREFERENCE_VALUE_AT_C1_K = 2.;
	RELATIVE_PREFERENCE_VALUE_AT_C1_L = 0.7;
	RELATIVE_PREFERENCE_VALUE_AT_C1_M = 1.;
	RELATIVE_PREFERENCE_VALUE_AT_C1_N = 1.2;
	RELATIVE_PREFERENCE_VALUE_AT_C1_P = 0.;
	RELATIVE_PREFERENCE_VALUE_AT_C1_Q = 2.1;
	RELATIVE_PREFERENCE_VALUE_AT_C1_R = 1.;
	RELATIVE_PREFERENCE_VALUE_AT_C1_S = 1.7;
	RELATIVE_PREFERENCE_VALUE_AT_C1_T = 1.;
	RELATIVE_PREFERENCE_VALUE_AT_C1_V = 0.7;
	RELATIVE_PREFERENCE_VALUE_AT_C1_W = 0.;
	RELATIVE_PREFERENCE_VALUE_AT_C1_Y = 1.2;

	//Correlation coefficient in regression analysis (Prabhakaran-Ponnuswamy, 1982)
	//AAINDEX = PRAM820103
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:PRAM820103
	CORRELATION_COEFFICIENT_IN_REGRESSION_ANALYSIS_A = 0.687;
	CORRELATION_COEFFICIENT_IN_REGRESSION_ANALYSIS_C = 0.263;
	CORRELATION_COEFFICIENT_IN_REGRESSION_ANALYSIS_D = 0.632;
	CORRELATION_COEFFICIENT_IN_REGRESSION_ANALYSIS_E = 0.669;
	CORRELATION_COEFFICIENT_IN_REGRESSION_ANALYSIS_F = 0.577;
	CORRELATION_COEFFICIENT_IN_REGRESSION_ANALYSIS_G = 0.670;
	CORRELATION_COEFFICIENT_IN_REGRESSION_ANALYSIS_H = 0.594;
	CORRELATION_COEFFICIENT_IN_REGRESSION_ANALYSIS_I = 0.564;
	CORRELATION_COEFFICIENT_IN_REGRESSION_ANALYSIS_K = 0.407;
	CORRELATION_COEFFICIENT_IN_REGRESSION_ANALYSIS_L = 0.541;
	CORRELATION_COEFFICIENT_IN_REGRESSION_ANALYSIS_M = 0.328;
	CORRELATION_COEFFICIENT_IN_REGRESSION_ANALYSIS_N = 0.489;
	CORRELATION_COEFFICIENT_IN_REGRESSION_ANALYSIS_P = 0.600;
	CORRELATION_COEFFICIENT_IN_REGRESSION_ANALYSIS_Q = 0.527;
	CORRELATION_COEFFICIENT_IN_REGRESSION_ANALYSIS_R = 0.590;
	CORRELATION_COEFFICIENT_IN_REGRESSION_ANALYSIS_S = 0.692;
	CORRELATION_COEFFICIENT_IN_REGRESSION_ANALYSIS_T = 0.713;
	CORRELATION_COEFFICIENT_IN_REGRESSION_ANALYSIS_V = 0.529;
	CORRELATION_COEFFICIENT_IN_REGRESSION_ANALYSIS_W = 0.632;
	CORRELATION_COEFFICIENT_IN_REGRESSION_ANALYSIS_Y = 0.495;

	//Weights for coil at the window position of 3 (Qian-Sejnowski, 1988)
	//AAINDEX = QIAN880136
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:QIAN880136
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_3_A = 0.01;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_3_C = -0.31;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_3_D = 0.78;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_3_E = 0.09;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_3_F = -0.20;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_3_G = 0.14;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_3_H = -0.56;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_3_I = -0.09;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_3_K = -0.43;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_3_L = -0.81;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_3_M = -0.49;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_3_N = -0.11;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_3_P = 1.14;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_3_Q = -0.13;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_3_R = -0.13;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_3_S = 0.13;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_3_T = -0.02;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_3_V = -0.11;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_3_W = -0.20;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_3_Y = 0.10;

	//Principal component III (Sneath, 1966)
	//AAINDEX = SNEP660103
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:SNEP660103
	PRINCIPAL_COMPONENT_III_A = -0.110;
	PRINCIPAL_COMPONENT_III_C = -0.184;
	PRINCIPAL_COMPONENT_III_D = -0.285;
	PRINCIPAL_COMPONENT_III_E = -0.246;
	PRINCIPAL_COMPONENT_III_F = 0.438;
	PRINCIPAL_COMPONENT_III_G = -0.073;
	PRINCIPAL_COMPONENT_III_H = 0.320;
	PRINCIPAL_COMPONENT_III_I = 0.001;
	PRINCIPAL_COMPONENT_III_K = 0.049;
	PRINCIPAL_COMPONENT_III_L = -0.008;
	PRINCIPAL_COMPONENT_III_M = -0.041;
	PRINCIPAL_COMPONENT_III_N = -0.136;
	PRINCIPAL_COMPONENT_III_P = -0.016;
	PRINCIPAL_COMPONENT_III_Q = -0.067;
	PRINCIPAL_COMPONENT_III_R = 0.079;
	PRINCIPAL_COMPONENT_III_S = -0.153;
	PRINCIPAL_COMPONENT_III_T = -0.208;
	PRINCIPAL_COMPONENT_III_V = -0.155;
	PRINCIPAL_COMPONENT_III_W = 0.493;
	PRINCIPAL_COMPONENT_III_Y = 0.381;

	//pK-N (Fasman, 1976)
	//AAINDEX = FASG760104
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:FASG760104
	PK_N_A = 9.69;
	PK_N_C = 8.35;
	PK_N_D = 9.60;
	PK_N_E = 9.67;
	PK_N_F = 9.18;
	PK_N_G = 9.78;
	PK_N_H = 9.17;
	PK_N_I = 9.68;
	PK_N_K = 9.18;
	PK_N_L = 9.60;
	PK_N_M = 9.21;
	PK_N_N = 8.80;
	PK_N_P = 10.64;
	PK_N_Q = 9.13;
	PK_N_R = 8.99;
	PK_N_S = 9.21;
	PK_N_T = 9.10;
	PK_N_V = 9.62;
	PK_N_W = 9.44;
	PK_N_Y = 9.11;

	//Free energy change of alpha(Ri) to alpha(Rh) (Wertz-Scheraga, 1978)
	//AAINDEX = WERD780103
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:WERD780103
	FREE_ENERGY_CHANGE_OF_ALPHARI_TO_ALPHARH_A = 0.15;
	FREE_ENERGY_CHANGE_OF_ALPHARI_TO_ALPHARH_C = -0.19;
	FREE_ENERGY_CHANGE_OF_ALPHARI_TO_ALPHARH_D = -0.22;
	FREE_ENERGY_CHANGE_OF_ALPHARI_TO_ALPHARH_E = 0.14;
	FREE_ENERGY_CHANGE_OF_ALPHARI_TO_ALPHARH_F = 1.18;
	FREE_ENERGY_CHANGE_OF_ALPHARI_TO_ALPHARH_G = 0.36;
	FREE_ENERGY_CHANGE_OF_ALPHARI_TO_ALPHARH_H = -0.25;
	FREE_ENERGY_CHANGE_OF_ALPHARI_TO_ALPHARH_I = 0.02;
	FREE_ENERGY_CHANGE_OF_ALPHARI_TO_ALPHARH_K = -0.16;
	FREE_ENERGY_CHANGE_OF_ALPHARI_TO_ALPHARH_L = 0.06;
	FREE_ENERGY_CHANGE_OF_ALPHARI_TO_ALPHARH_M = 0.11;
	FREE_ENERGY_CHANGE_OF_ALPHARI_TO_ALPHARH_N = 0.69;
	FREE_ENERGY_CHANGE_OF_ALPHARI_TO_ALPHARH_P = 0.11;
	FREE_ENERGY_CHANGE_OF_ALPHARI_TO_ALPHARH_Q = -0.06;
	FREE_ENERGY_CHANGE_OF_ALPHARI_TO_ALPHARH_R = -0.37;
	FREE_ENERGY_CHANGE_OF_ALPHARI_TO_ALPHARH_S = 0.13;
	FREE_ENERGY_CHANGE_OF_ALPHARI_TO_ALPHARH_T = 0.28;
	FREE_ENERGY_CHANGE_OF_ALPHARI_TO_ALPHARH_V = -0.08;
	FREE_ENERGY_CHANGE_OF_ALPHARI_TO_ALPHARH_W = -0.12;
	FREE_ENERGY_CHANGE_OF_ALPHARI_TO_ALPHARH_Y = 0.19;

	//Information measure for N-terminal helix (Robson-Suzuki, 1976)
	//AAINDEX = ROBB760102
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:ROBB760102
	INFORMATION_MEASURE_FOR_N_TERMINAL_HELIX_A = 2.3;
	INFORMATION_MEASURE_FOR_N_TERMINAL_HELIX_C = 0.8;
	INFORMATION_MEASURE_FOR_N_TERMINAL_HELIX_D = 7.4;
	INFORMATION_MEASURE_FOR_N_TERMINAL_HELIX_E = 10.3;
	INFORMATION_MEASURE_FOR_N_TERMINAL_HELIX_F = -1.1;
	INFORMATION_MEASURE_FOR_N_TERMINAL_HELIX_G = -5.2;
	INFORMATION_MEASURE_FOR_N_TERMINAL_HELIX_H = -2.8;
	INFORMATION_MEASURE_FOR_N_TERMINAL_HELIX_I = -4.0;
	INFORMATION_MEASURE_FOR_N_TERMINAL_HELIX_K = -4.1;
	INFORMATION_MEASURE_FOR_N_TERMINAL_HELIX_L = -2.1;
	INFORMATION_MEASURE_FOR_N_TERMINAL_HELIX_M = -3.5;
	INFORMATION_MEASURE_FOR_N_TERMINAL_HELIX_N = 0.3;
	INFORMATION_MEASURE_FOR_N_TERMINAL_HELIX_P = 8.1;
	INFORMATION_MEASURE_FOR_N_TERMINAL_HELIX_Q = -0.7;
	INFORMATION_MEASURE_FOR_N_TERMINAL_HELIX_R = -5.2;
	INFORMATION_MEASURE_FOR_N_TERMINAL_HELIX_S = -3.5;
	INFORMATION_MEASURE_FOR_N_TERMINAL_HELIX_T = 2.3;
	INFORMATION_MEASURE_FOR_N_TERMINAL_HELIX_V = -4.4;
	INFORMATION_MEASURE_FOR_N_TERMINAL_HELIX_W = -0.9;
	INFORMATION_MEASURE_FOR_N_TERMINAL_HELIX_Y = -3.7;

	//Unfolding Gibbs energy in water, pH9.0 (Yutani et al., 1987)
	//AAINDEX = YUTK870102
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:YUTK870102
	UNFOLDING_GIBBS_ENERGY_IN_WATER_PH9_A = 6.8;
	UNFOLDING_GIBBS_ENERGY_IN_WATER_PH9_C = 8.3;
	UNFOLDING_GIBBS_ENERGY_IN_WATER_PH9_D = 7.0;
	UNFOLDING_GIBBS_ENERGY_IN_WATER_PH9_E = 4.9;
	UNFOLDING_GIBBS_ENERGY_IN_WATER_PH9_F = 8.3;
	UNFOLDING_GIBBS_ENERGY_IN_WATER_PH9_G = 6.4;
	UNFOLDING_GIBBS_ENERGY_IN_WATER_PH9_H = 9.2;
	UNFOLDING_GIBBS_ENERGY_IN_WATER_PH9_I = 10.0;
	UNFOLDING_GIBBS_ENERGY_IN_WATER_PH9_K = 7.5;
	UNFOLDING_GIBBS_ENERGY_IN_WATER_PH9_L = 12.2;
	UNFOLDING_GIBBS_ENERGY_IN_WATER_PH9_M = 8.4;
	UNFOLDING_GIBBS_ENERGY_IN_WATER_PH9_N = 6.2;
	UNFOLDING_GIBBS_ENERGY_IN_WATER_PH9_P = 6.9;
	UNFOLDING_GIBBS_ENERGY_IN_WATER_PH9_Q = 8.5;
	UNFOLDING_GIBBS_ENERGY_IN_WATER_PH9_R = 0.;
	UNFOLDING_GIBBS_ENERGY_IN_WATER_PH9_S = 8.0;
	UNFOLDING_GIBBS_ENERGY_IN_WATER_PH9_T = 7.0;
	UNFOLDING_GIBBS_ENERGY_IN_WATER_PH9_V = 9.4;
	UNFOLDING_GIBBS_ENERGY_IN_WATER_PH9_W = 5.7;
	UNFOLDING_GIBBS_ENERGY_IN_WATER_PH9_Y = 6.8;

	//Average relative fractional occurrence in AL(i) (Rackovsky-Scheraga, 1982)
	//AAINDEX = RACS820103
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:RACS820103
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALI_A = 0.82;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALI_C = 0.00;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALI_D = 2.64;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALI_E = 2.62;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALI_F = 0.00;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALI_G = 1.63;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALI_H = 0.00;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALI_I = 2.32;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALI_K = 2.86;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALI_L = 0.00;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALI_M = 0.00;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALI_N = 2.07;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALI_P = 0.00;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALI_Q = 0.00;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALI_R = 2.60;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALI_S = 1.23;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALI_T = 2.48;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALI_V = 1.62;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALI_W = 0.00;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALI_Y = 1.90;

	//Weights for coil at the window position of 6 (Qian-Sejnowski, 1988)
	//QIAN880139
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:QIAN880139
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_6_A = 0.08;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_6_C = 0.37;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_6_D = 0.04;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_6_E = 0.36;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_6_F = 0.34;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_6_G = -0.02;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_6_H = -0.45;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_6_I = 0.09;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_6_K = -0.27;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_6_L = 0.24;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_6_M = 0.16;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_6_N = -0.06;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_6_P = 0.16;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_6_Q = 0.48;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_6_R = -0.01;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_6_S = -0.35;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_6_T = -0.04;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_6_V = 0.18;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_6_W = -0.06;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_6_Y = -0.20;

	//Optimized propensity to form reverse turn (Oobatake et al., 1985)
	//AAINEX = OOBM850102
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:OOBM850102
	OPTIMIZED_PROPENSITY_TO_FORM_REVERSE_TURN_A = 1.34;
	OPTIMIZED_PROPENSITY_TO_FORM_REVERSE_TURN_C = 1.07;
	OPTIMIZED_PROPENSITY_TO_FORM_REVERSE_TURN_D = 3.32;
	OPTIMIZED_PROPENSITY_TO_FORM_REVERSE_TURN_E = 2.20;
	OPTIMIZED_PROPENSITY_TO_FORM_REVERSE_TURN_F = 0.80;
	OPTIMIZED_PROPENSITY_TO_FORM_REVERSE_TURN_G = 2.07;
	OPTIMIZED_PROPENSITY_TO_FORM_REVERSE_TURN_H = 1.27;
	OPTIMIZED_PROPENSITY_TO_FORM_REVERSE_TURN_I = 0.66;
	OPTIMIZED_PROPENSITY_TO_FORM_REVERSE_TURN_K = 0.61;
	OPTIMIZED_PROPENSITY_TO_FORM_REVERSE_TURN_L = 0.54;
	OPTIMIZED_PROPENSITY_TO_FORM_REVERSE_TURN_M = 0.70;
	OPTIMIZED_PROPENSITY_TO_FORM_REVERSE_TURN_N = 2.49;
	OPTIMIZED_PROPENSITY_TO_FORM_REVERSE_TURN_P = 2.12;
	OPTIMIZED_PROPENSITY_TO_FORM_REVERSE_TURN_Q = 1.49;
	OPTIMIZED_PROPENSITY_TO_FORM_REVERSE_TURN_R = 0.95;
	OPTIMIZED_PROPENSITY_TO_FORM_REVERSE_TURN_S = 0.94;
	OPTIMIZED_PROPENSITY_TO_FORM_REVERSE_TURN_T = 1.09;
	OPTIMIZED_PROPENSITY_TO_FORM_REVERSE_TURN_V = 1.32;
	OPTIMIZED_PROPENSITY_TO_FORM_REVERSE_TURN_W = -4.65;
	OPTIMIZED_PROPENSITY_TO_FORM_REVERSE_TURN_Y = -0.17;

	//Information measure for C-terminal turn (Robson-Suzuki, 1976) 
	//AAINDEX = ROBB760111
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:ROBB760111
	INFORMATION_MEASURE_FOR_C_TERMINAL_TURN_A = -3.7;
	INFORMATION_MEASURE_FOR_C_TERMINAL_TURN_C = 4.0;
	INFORMATION_MEASURE_FOR_C_TERMINAL_TURN_D = -0.6;
	INFORMATION_MEASURE_FOR_C_TERMINAL_TURN_E = -4.3;
	INFORMATION_MEASURE_FOR_C_TERMINAL_TURN_F = 1.6;
	INFORMATION_MEASURE_FOR_C_TERMINAL_TURN_G = 5.9;
	INFORMATION_MEASURE_FOR_C_TERMINAL_TURN_H = -0.8;
	INFORMATION_MEASURE_FOR_C_TERMINAL_TURN_I = -0.5;
	INFORMATION_MEASURE_FOR_C_TERMINAL_TURN_K = 1.3;
	INFORMATION_MEASURE_FOR_C_TERMINAL_TURN_L = -2.8;
	INFORMATION_MEASURE_FOR_C_TERMINAL_TURN_M = -1.6;
	INFORMATION_MEASURE_FOR_C_TERMINAL_TURN_N = -0.6;
	INFORMATION_MEASURE_FOR_C_TERMINAL_TURN_P = -6.0;
	INFORMATION_MEASURE_FOR_C_TERMINAL_TURN_Q = 3.4;
	INFORMATION_MEASURE_FOR_C_TERMINAL_TURN_R = 1.0;
	INFORMATION_MEASURE_FOR_C_TERMINAL_TURN_S = 1.5;
	INFORMATION_MEASURE_FOR_C_TERMINAL_TURN_T = 1.2;
	INFORMATION_MEASURE_FOR_C_TERMINAL_TURN_V = -4.6;
	INFORMATION_MEASURE_FOR_C_TERMINAL_TURN_W = 6.5;
	INFORMATION_MEASURE_FOR_C_TERMINAL_TURN_Y = 1.3;

	//Composition of amino acids in nuclear proteins (percent) (Cedano et al., 1997)
	//AAINDEX = CEDJ970105
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:CEDJ970105
	COMPOSITION_OF_AA_IN_NUCLEAR_PROTEINS_PERCENT_A = 8.3;
	COMPOSITION_OF_AA_IN_NUCLEAR_PROTEINS_PERCENT_C = 1.6;
	COMPOSITION_OF_AA_IN_NUCLEAR_PROTEINS_PERCENT_D = 4.7;
	COMPOSITION_OF_AA_IN_NUCLEAR_PROTEINS_PERCENT_E = 6.5;
	COMPOSITION_OF_AA_IN_NUCLEAR_PROTEINS_PERCENT_F = 2.7;
	COMPOSITION_OF_AA_IN_NUCLEAR_PROTEINS_PERCENT_G = 6.3;
	COMPOSITION_OF_AA_IN_NUCLEAR_PROTEINS_PERCENT_H = 2.1;
	COMPOSITION_OF_AA_IN_NUCLEAR_PROTEINS_PERCENT_I = 3.7;
	COMPOSITION_OF_AA_IN_NUCLEAR_PROTEINS_PERCENT_K = 7.9;
	COMPOSITION_OF_AA_IN_NUCLEAR_PROTEINS_PERCENT_L = 7.4;
	COMPOSITION_OF_AA_IN_NUCLEAR_PROTEINS_PERCENT_M = 2.3;
	COMPOSITION_OF_AA_IN_NUCLEAR_PROTEINS_PERCENT_N = 3.7;
	COMPOSITION_OF_AA_IN_NUCLEAR_PROTEINS_PERCENT_P = 6.9;
	COMPOSITION_OF_AA_IN_NUCLEAR_PROTEINS_PERCENT_Q = 4.7;
	COMPOSITION_OF_AA_IN_NUCLEAR_PROTEINS_PERCENT_R = 8.7;
	COMPOSITION_OF_AA_IN_NUCLEAR_PROTEINS_PERCENT_S = 8.8;
	COMPOSITION_OF_AA_IN_NUCLEAR_PROTEINS_PERCENT_T = 5.1;
	COMPOSITION_OF_AA_IN_NUCLEAR_PROTEINS_PERCENT_V = 5.3;
	COMPOSITION_OF_AA_IN_NUCLEAR_PROTEINS_PERCENT_W = 0.7;
	COMPOSITION_OF_AA_IN_NUCLEAR_PROTEINS_PERCENT_Y = 2.4;

	//Optimized average non-bonded energy per atom (Oobatake et al., 1985)
	//AAINDEX = OOBM850104
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:OOBM850104
	OPTIMIZED_AVERAGE_NON_BONDED_ENERGY_PER_ATOM_A = -2.49;
	OPTIMIZED_AVERAGE_NON_BONDED_ENERGY_PER_ATOM_C = -3.13;
	OPTIMIZED_AVERAGE_NON_BONDED_ENERGY_PER_ATOM_D = 8.86;
	OPTIMIZED_AVERAGE_NON_BONDED_ENERGY_PER_ATOM_E = 4.04;
	OPTIMIZED_AVERAGE_NON_BONDED_ENERGY_PER_ATOM_F = -6.64;
	OPTIMIZED_AVERAGE_NON_BONDED_ENERGY_PER_ATOM_G = -0.56;
	OPTIMIZED_AVERAGE_NON_BONDED_ENERGY_PER_ATOM_H = 4.22;
	OPTIMIZED_AVERAGE_NON_BONDED_ENERGY_PER_ATOM_I = -10.87;
	OPTIMIZED_AVERAGE_NON_BONDED_ENERGY_PER_ATOM_K = -9.97;
	OPTIMIZED_AVERAGE_NON_BONDED_ENERGY_PER_ATOM_L = -7.16;
	OPTIMIZED_AVERAGE_NON_BONDED_ENERGY_PER_ATOM_M = -4.96;
	OPTIMIZED_AVERAGE_NON_BONDED_ENERGY_PER_ATOM_N = 2.27;
	OPTIMIZED_AVERAGE_NON_BONDED_ENERGY_PER_ATOM_P = 5.19;
	OPTIMIZED_AVERAGE_NON_BONDED_ENERGY_PER_ATOM_Q = 1.79;
	OPTIMIZED_AVERAGE_NON_BONDED_ENERGY_PER_ATOM_R = 2.55;
	OPTIMIZED_AVERAGE_NON_BONDED_ENERGY_PER_ATOM_S = -1.60;
	OPTIMIZED_AVERAGE_NON_BONDED_ENERGY_PER_ATOM_T = -4.75;
	OPTIMIZED_AVERAGE_NON_BONDED_ENERGY_PER_ATOM_V = -3.97;
	OPTIMIZED_AVERAGE_NON_BONDED_ENERGY_PER_ATOM_W = -17.84;
	OPTIMIZED_AVERAGE_NON_BONDED_ENERGY_PER_ATOM_Y = 9.25;

	//AA composition of mt-proteins from animal (Nakashima et al., 1990)
	//AAINDEX = NAKH900105
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:NAKH900105
	AA_COMPOSITION_OF_MT_PROTEINS_FROM_ANIMAL_A = 5.88;
	AA_COMPOSITION_OF_MT_PROTEINS_FROM_ANIMAL_C = 1.11;
	AA_COMPOSITION_OF_MT_PROTEINS_FROM_ANIMAL_D = 1.70;
	AA_COMPOSITION_OF_MT_PROTEINS_FROM_ANIMAL_E = 2.60;
	AA_COMPOSITION_OF_MT_PROTEINS_FROM_ANIMAL_F = 6.58;
	AA_COMPOSITION_OF_MT_PROTEINS_FROM_ANIMAL_G = 5.29;
	AA_COMPOSITION_OF_MT_PROTEINS_FROM_ANIMAL_H = 2.33;
	AA_COMPOSITION_OF_MT_PROTEINS_FROM_ANIMAL_I = 8.78;
	AA_COMPOSITION_OF_MT_PROTEINS_FROM_ANIMAL_K = 2.58;
	AA_COMPOSITION_OF_MT_PROTEINS_FROM_ANIMAL_L = 16.52;
	AA_COMPOSITION_OF_MT_PROTEINS_FROM_ANIMAL_M = 6.00;
	AA_COMPOSITION_OF_MT_PROTEINS_FROM_ANIMAL_N = 4.38;
	AA_COMPOSITION_OF_MT_PROTEINS_FROM_ANIMAL_P = 5.29;
	AA_COMPOSITION_OF_MT_PROTEINS_FROM_ANIMAL_Q = 2.30;
	AA_COMPOSITION_OF_MT_PROTEINS_FROM_ANIMAL_R = 1.54;
	AA_COMPOSITION_OF_MT_PROTEINS_FROM_ANIMAL_S = 7.68;
	AA_COMPOSITION_OF_MT_PROTEINS_FROM_ANIMAL_T = 8.38;
	AA_COMPOSITION_OF_MT_PROTEINS_FROM_ANIMAL_V = 4.66;
	AA_COMPOSITION_OF_MT_PROTEINS_FROM_ANIMAL_W = 2.89;
	AA_COMPOSITION_OF_MT_PROTEINS_FROM_ANIMAL_Y = 3.51;

	//Average relative fractional occurrence in AL(i-1) (Rackovsky-Scheraga, 1982)
	//AAINDEX = RACS820109
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:RACS820109
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALiMinus1_A = 0.00;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALiMinus1_C = 0.00;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALiMinus1_D = 2.15;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALiMinus1_E = 0.00;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALiMinus1_F = 2.11;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALiMinus1_G = 6.49;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALiMinus1_H = 0.00;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALiMinus1_I = 0.00;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALiMinus1_K = 0.00;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALiMinus1_L = 0.00;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALiMinus1_M = 0.00;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALiMinus1_N = 4.14;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALiMinus1_P = 1.99;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALiMinus1_Q = 0.00;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALiMinus1_R = 0.00;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALiMinus1_S = 0.00;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALiMinus1_T = 1.24;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALiMinus1_V = 0.00;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALiMinus1_W = 0.00;
	AVERAGE_RELATIVE_FRACTIONAL_OCCURENCE_IN_ALiMinus1_Y = 1.90;

	//Free energy in beta-strand region (Munoz-Serrano, 1994)
	//AAINDEX = MUNV940105
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:MUNV940105
	FREE_ENERGY_IN_BETA_STRAND_REGION_A = 1.40;
	FREE_ENERGY_IN_BETA_STRAND_REGION_C = 1.14;
	FREE_ENERGY_IN_BETA_STRAND_REGION_D = 1.89;
	FREE_ENERGY_IN_BETA_STRAND_REGION_E = 1.42;
	FREE_ENERGY_IN_BETA_STRAND_REGION_F = 1.07;
	FREE_ENERGY_IN_BETA_STRAND_REGION_G = 2.06;
	FREE_ENERGY_IN_BETA_STRAND_REGION_H = 1.25;
	FREE_ENERGY_IN_BETA_STRAND_REGION_I = 1.02;
	FREE_ENERGY_IN_BETA_STRAND_REGION_K = 1.34;
	FREE_ENERGY_IN_BETA_STRAND_REGION_L = 1.33;
	FREE_ENERGY_IN_BETA_STRAND_REGION_M = 1.12;
	FREE_ENERGY_IN_BETA_STRAND_REGION_N = 1.61;
	FREE_ENERGY_IN_BETA_STRAND_REGION_P = 3.90;
	FREE_ENERGY_IN_BETA_STRAND_REGION_Q = 1.33;
	FREE_ENERGY_IN_BETA_STRAND_REGION_R = 1.23;
	FREE_ENERGY_IN_BETA_STRAND_REGION_S = 1.20;
	FREE_ENERGY_IN_BETA_STRAND_REGION_T = 0.99;
	FREE_ENERGY_IN_BETA_STRAND_REGION_V = 0.87;
	FREE_ENERGY_IN_BETA_STRAND_REGION_W = 1.10;
	FREE_ENERGY_IN_BETA_STRAND_REGION_Y = 0.98;

	//Slopes dekapeptide, FDPB VFF neutral (Avbelj, 2000)
	//AAINDEX = AVBF000108 
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:AVBF000108
	SLOPES_DEKAPEPTIDE_FDPB_VFF_NEUTRAL_A = -0.623;
	SLOPES_DEKAPEPTIDE_FDPB_VFF_NEUTRAL_C = -0.571;
	SLOPES_DEKAPEPTIDE_FDPB_VFF_NEUTRAL_D = -0.626;
	SLOPES_DEKAPEPTIDE_FDPB_VFF_NEUTRAL_E = -0.572;
	SLOPES_DEKAPEPTIDE_FDPB_VFF_NEUTRAL_F = -0.461;
	SLOPES_DEKAPEPTIDE_FDPB_VFF_NEUTRAL_G = -0.679;
	SLOPES_DEKAPEPTIDE_FDPB_VFF_NEUTRAL_H = -0.508;
	SLOPES_DEKAPEPTIDE_FDPB_VFF_NEUTRAL_I = -0.199;
	SLOPES_DEKAPEPTIDE_FDPB_VFF_NEUTRAL_K = -0.581;
	SLOPES_DEKAPEPTIDE_FDPB_VFF_NEUTRAL_L = -0.527;
	SLOPES_DEKAPEPTIDE_FDPB_VFF_NEUTRAL_M = -0.571;
	SLOPES_DEKAPEPTIDE_FDPB_VFF_NEUTRAL_N = -0.619;

	//please pay attention to the variabler below, in the website it is denoted as NA, i.e. Not Available
	//we assin it the mean of all the other 19 variables. -9.395/19=-0.494
	SLOPES_DEKAPEPTIDE_FDPB_VFF_NEUTRAL_P = -0.494;

	SLOPES_DEKAPEPTIDE_FDPB_VFF_NEUTRAL_Q = -0.559;
	SLOPES_DEKAPEPTIDE_FDPB_VFF_NEUTRAL_R = -0.567;
	SLOPES_DEKAPEPTIDE_FDPB_VFF_NEUTRAL_S = -0.458;
	SLOPES_DEKAPEPTIDE_FDPB_VFF_NEUTRAL_T = -0.233;
	SLOPES_DEKAPEPTIDE_FDPB_VFF_NEUTRAL_V = -0.263;
	SLOPES_DEKAPEPTIDE_FDPB_VFF_NEUTRAL_W = -0.327;
	SLOPES_DEKAPEPTIDE_FDPB_VFF_NEUTRAL_Y = -0.451;

	//Weights for coil at the window position of -4 (Qian-Sejnowski, 1988)
	//AAINDEX = QIAN880129
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:QIAN880129
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS4_A = -0.43;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS4_C = 0.19;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS4_D = -0.31;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS4_E = -0.41;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS4_F = 0.25;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS4_G = -0.21;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS4_H = 0.21;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS4_I = 0.29;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS4_K = 0.33;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS4_L = -0.10;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS4_M = -0.01;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS4_N = 0.00;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS4_P = 0.28;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS4_Q = 0.14;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS4_R = 0.06;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS4_S = -0.23;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS4_T = -0.26;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS4_V = -0.10;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS4_W = 0.15;
	WEIGHTS_FOR_COIL_AT_THE_WINDOW_POSITION_OF_MINUS4_Y = 0.09;

	//A parameter defined from the residuals obtained from the best correlation of the Chou-Fasman parameter of beta-sheet (Charton-Charton, 1983)
	//AAINDEX = CHAM830102
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:CHAM830102
	A_PARA_DEF_FROM_THE_RESIDUALS_OBTAINED_FROM_THE_BEST_CORRELATION_OF_CF_PARA_OF_BETA_SHEET_A = -0.118;
	A_PARA_DEF_FROM_THE_RESIDUALS_OBTAINED_FROM_THE_BEST_CORRELATION_OF_CF_PARA_OF_BETA_SHEET_C = 0.083;
	A_PARA_DEF_FROM_THE_RESIDUALS_OBTAINED_FROM_THE_BEST_CORRELATION_OF_CF_PARA_OF_BETA_SHEET_D = 0.048;
	A_PARA_DEF_FROM_THE_RESIDUALS_OBTAINED_FROM_THE_BEST_CORRELATION_OF_CF_PARA_OF_BETA_SHEET_E = -0.245;
	A_PARA_DEF_FROM_THE_RESIDUALS_OBTAINED_FROM_THE_BEST_CORRELATION_OF_CF_PARA_OF_BETA_SHEET_F = 0.015;
	A_PARA_DEF_FROM_THE_RESIDUALS_OBTAINED_FROM_THE_BEST_CORRELATION_OF_CF_PARA_OF_BETA_SHEET_G = 0.104;
	A_PARA_DEF_FROM_THE_RESIDUALS_OBTAINED_FROM_THE_BEST_CORRELATION_OF_CF_PARA_OF_BETA_SHEET_H = 0.138;
	A_PARA_DEF_FROM_THE_RESIDUALS_OBTAINED_FROM_THE_BEST_CORRELATION_OF_CF_PARA_OF_BETA_SHEET_I = 0.230;
	A_PARA_DEF_FROM_THE_RESIDUALS_OBTAINED_FROM_THE_BEST_CORRELATION_OF_CF_PARA_OF_BETA_SHEET_K = 0.032;
	A_PARA_DEF_FROM_THE_RESIDUALS_OBTAINED_FROM_THE_BEST_CORRELATION_OF_CF_PARA_OF_BETA_SHEET_L = -0.052;
	A_PARA_DEF_FROM_THE_RESIDUALS_OBTAINED_FROM_THE_BEST_CORRELATION_OF_CF_PARA_OF_BETA_SHEET_M = -0.258;
	A_PARA_DEF_FROM_THE_RESIDUALS_OBTAINED_FROM_THE_BEST_CORRELATION_OF_CF_PARA_OF_BETA_SHEET_N = 0.289;
	A_PARA_DEF_FROM_THE_RESIDUALS_OBTAINED_FROM_THE_BEST_CORRELATION_OF_CF_PARA_OF_BETA_SHEET_P = 0.;
	A_PARA_DEF_FROM_THE_RESIDUALS_OBTAINED_FROM_THE_BEST_CORRELATION_OF_CF_PARA_OF_BETA_SHEET_Q = -0.105;
	A_PARA_DEF_FROM_THE_RESIDUALS_OBTAINED_FROM_THE_BEST_CORRELATION_OF_CF_PARA_OF_BETA_SHEET_R = 0.124;
	A_PARA_DEF_FROM_THE_RESIDUALS_OBTAINED_FROM_THE_BEST_CORRELATION_OF_CF_PARA_OF_BETA_SHEET_S = 0.225;
	A_PARA_DEF_FROM_THE_RESIDUALS_OBTAINED_FROM_THE_BEST_CORRELATION_OF_CF_PARA_OF_BETA_SHEET_T = 0.166;
	A_PARA_DEF_FROM_THE_RESIDUALS_OBTAINED_FROM_THE_BEST_CORRELATION_OF_CF_PARA_OF_BETA_SHEET_V = 0.513;
	A_PARA_DEF_FROM_THE_RESIDUALS_OBTAINED_FROM_THE_BEST_CORRELATION_OF_CF_PARA_OF_BETA_SHEET_W = 0.158;
	A_PARA_DEF_FROM_THE_RESIDUALS_OBTAINED_FROM_THE_BEST_CORRELATION_OF_CF_PARA_OF_BETA_SHEET_Y = 0.094;

	/*---------------below are from 12/16/2013-----------------------------------*/
	//Normalized flexibility parameters (B-values), average (Vihinen et al., 1994)
	//AAINDEX=VINM940101
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:VINM940101
	Normalized_Flexibility_Parameters_B_Values_A = 0.984;
	Normalized_Flexibility_Parameters_B_Values_C = 0.906;
	Normalized_Flexibility_Parameters_B_Values_D = 1.068;
	Normalized_Flexibility_Parameters_B_Values_E = 1.094;
	Normalized_Flexibility_Parameters_B_Values_F = 0.915;
	Normalized_Flexibility_Parameters_B_Values_G = 1.031;
	Normalized_Flexibility_Parameters_B_Values_H = 0.950;
	Normalized_Flexibility_Parameters_B_Values_I = 0.927;
	Normalized_Flexibility_Parameters_B_Values_K = 1.102;
	Normalized_Flexibility_Parameters_B_Values_L = 0.935;
	Normalized_Flexibility_Parameters_B_Values_M = 0.952;
	Normalized_Flexibility_Parameters_B_Values_N = 1.048;
	Normalized_Flexibility_Parameters_B_Values_P = 1.049;
	Normalized_Flexibility_Parameters_B_Values_Q = 1.037;
	Normalized_Flexibility_Parameters_B_Values_R = 1.008;
	Normalized_Flexibility_Parameters_B_Values_S = 1.046;
	Normalized_Flexibility_Parameters_B_Values_T = 0.997;
	Normalized_Flexibility_Parameters_B_Values_V = 0.931;
	Normalized_Flexibility_Parameters_B_Values_W = 0.904;
	Normalized_Flexibility_Parameters_B_Values_Y = 0.929;

	//Normalized van der Waals volume (Fauchere et al., 1988)
	//AAINDEX = FAUJ880103
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:FAUJ880103
	Normalized_Van_Der_Waals_Volume_A = 1.00;
	Normalized_Van_Der_Waals_Volume_C = 2.43;
	Normalized_Van_Der_Waals_Volume_D = 2.78;
	Normalized_Van_Der_Waals_Volume_E = 3.78;
	Normalized_Van_Der_Waals_Volume_F = 5.89;
	Normalized_Van_Der_Waals_Volume_G = 0.00;
	Normalized_Van_Der_Waals_Volume_H = 4.66;
	Normalized_Van_Der_Waals_Volume_I = 4.00;
	Normalized_Van_Der_Waals_Volume_K = 4.77;
	Normalized_Van_Der_Waals_Volume_L = 4.00;
	Normalized_Van_Der_Waals_Volume_M = 4.43;
	Normalized_Van_Der_Waals_Volume_N = 2.95;
	Normalized_Van_Der_Waals_Volume_P = 2.72;
	Normalized_Van_Der_Waals_Volume_Q = 3.95;
	Normalized_Van_Der_Waals_Volume_R = 6.13;
	Normalized_Van_Der_Waals_Volume_S = 1.60;
	Normalized_Van_Der_Waals_Volume_T = 2.60;
	Normalized_Van_Der_Waals_Volume_V = 3.00;
	Normalized_Van_Der_Waals_Volume_W = 8.08;
	Normalized_Van_Der_Waals_Volume_Y = 6.47;

	//RF value in high salt chromatography (Weber-Lacey, 1978)
	//AAINDEX=WEBA780101
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:WEBA780101
	RF_Value_In_High_Salt_Chromatography_A = 0.89;
	RF_Value_In_High_Salt_Chromatography_C = 0.85;
	RF_Value_In_High_Salt_Chromatography_D = 0.87;
	RF_Value_In_High_Salt_Chromatography_E = 0.84;
	RF_Value_In_High_Salt_Chromatography_F = 0.52;
	RF_Value_In_High_Salt_Chromatography_G = 0.92;
	RF_Value_In_High_Salt_Chromatography_H = 0.83;
	RF_Value_In_High_Salt_Chromatography_I = 0.76;
	RF_Value_In_High_Salt_Chromatography_K = 0.97;
	RF_Value_In_High_Salt_Chromatography_L = 0.73;
	RF_Value_In_High_Salt_Chromatography_M = 0.74;
	RF_Value_In_High_Salt_Chromatography_N = 0.89;
	RF_Value_In_High_Salt_Chromatography_P = 0.82;
	RF_Value_In_High_Salt_Chromatography_Q = 0.82;
	RF_Value_In_High_Salt_Chromatography_R = 0.88;
	RF_Value_In_High_Salt_Chromatography_S = 0.96;
	RF_Value_In_High_Salt_Chromatography_T = 0.92;
	RF_Value_In_High_Salt_Chromatography_V = 0.85;
	RF_Value_In_High_Salt_Chromatography_W = 0.20;
	RF_Value_In_High_Salt_Chromatography_Y = 0.49;

	//Isoelectric point (Zimmerman et al., 1968)
	//AAINDEX = ZIMJ680104
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:ZIMJ680104
	Isoelectric_Point_A = 6.00;
	Isoelectric_Point_C = 5.05;
	Isoelectric_Point_D = 2.77;
	Isoelectric_Point_E = 3.22;
	Isoelectric_Point_F = 5.48;
	Isoelectric_Point_G = 5.97;
	Isoelectric_Point_H = 7.59;
	Isoelectric_Point_I = 6.02;
	Isoelectric_Point_K = 9.74;
	Isoelectric_Point_L = 5.98;
	Isoelectric_Point_M = 5.74;
	Isoelectric_Point_N = 5.41;
	Isoelectric_Point_P = 6.30;
	Isoelectric_Point_Q = 5.65;
	Isoelectric_Point_R = 10.76;
	Isoelectric_Point_S = 5.68;
	Isoelectric_Point_T = 5.66;
	Isoelectric_Point_V = 5.96;
	Isoelectric_Point_W = 5.89;
	Isoelectric_Point_Y = 5.66;

	//Average surrounding hydrophobicity (Manavalan-Ponnuswamy, 1978)
	//AAINDEX = MANP780101
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:MANP780101
	Average_Surrounding_Hydrophobicity_A = 12.97;
	Average_Surrounding_Hydrophobicity_C = 14.63;
	Average_Surrounding_Hydrophobicity_D = 10.85;
	Average_Surrounding_Hydrophobicity_E = 11.89;
	Average_Surrounding_Hydrophobicity_F = 14.00;
	Average_Surrounding_Hydrophobicity_G = 12.43;
	Average_Surrounding_Hydrophobicity_H = 12.16;
	Average_Surrounding_Hydrophobicity_I = 15.67;
	Average_Surrounding_Hydrophobicity_K = 11.36;
	Average_Surrounding_Hydrophobicity_L = 14.90;
	Average_Surrounding_Hydrophobicity_M = 14.39;
	Average_Surrounding_Hydrophobicity_N = 11.42;
	Average_Surrounding_Hydrophobicity_P = 11.37;
	Average_Surrounding_Hydrophobicity_Q = 11.76;
	Average_Surrounding_Hydrophobicity_R = 11.72;
	Average_Surrounding_Hydrophobicity_S = 11.23;
	Average_Surrounding_Hydrophobicity_T = 11.69;
	Average_Surrounding_Hydrophobicity_V = 15.71;
	Average_Surrounding_Hydrophobicity_W = 13.93;
	Average_Surrounding_Hydrophobicity_Y = 13.42;

	//Retention coefficient at pH 2 (Guo et al., 1986)
	//AAINDEX = GUOD860101;
	//http://www.genome.jp/dbget-bin/www_bget?aaindex:GUOD860101
	Retention_Coefficient_At_Ph_2_A = 25;
	Retention_Coefficient_At_Ph_2_C = 32;
	Retention_Coefficient_At_Ph_2_D = 2;
	Retention_Coefficient_At_Ph_2_E = 14;
	Retention_Coefficient_At_Ph_2_F = 100;
	Retention_Coefficient_At_Ph_2_G = -2;
	Retention_Coefficient_At_Ph_2_H = -26;
	Retention_Coefficient_At_Ph_2_I = 91;
	Retention_Coefficient_At_Ph_2_K = -26;
	Retention_Coefficient_At_Ph_2_L = 100;
	Retention_Coefficient_At_Ph_2_M = 68;
	Retention_Coefficient_At_Ph_2_N = -7;
	Retention_Coefficient_At_Ph_2_P = 25;
	Retention_Coefficient_At_Ph_2_Q = 0;
	Retention_Coefficient_At_Ph_2_R = -7;
	Retention_Coefficient_At_Ph_2_S = -2;
	Retention_Coefficient_At_Ph_2_T = 7;
	Retention_Coefficient_At_Ph_2_V = 62;
	Retention_Coefficient_At_Ph_2_W = 109;
	Retention_Coefficient_At_Ph_2_Y = 56;

}


Constants::~Constants(void)
{
}
